import config, utils, sys

non_nouns = ['VB', 'VBZ', 'IN', 'DT', 'WDT', 'CD']

class Word:

	def __init__ (self, word):
		self.word = word.lower ()
		self.stem = utils.stem_word (self.word)
		self.pos_tag = None

	def __str__ (self):
		return self.word

	def __eq__ (self, other_word):
		return other_word.word == self.word and other_word.pos_tag == self.pos_tag

	def __hash__ (self):
		return int ((self.word.__hash__ () * self.pos_tag.__hash__ () * self.stem.__hash__ ()) & sys.maxint)
	
START_WORD = Word ("#start#")
START_WORD.pos_tag = START_WORD.word

END_WORD = Word ("#end#")
END_WORD.pos_tag = END_WORD.word


class Sentence:

	def __init__ (self, text):

		self.text = utils.tokenize_text (utils.clean_sentence (text))
		self.words = []
		self.stems = set ()

		for token in utils.split_into_words (self.text):
			word = Word (token)
			self.words.append (word)
			self.stems.add (word.stem)

	def get_tagged_text (self):
		twords = self.text.lower ().split (" ")
		tagged_text = ""

		word_index = 0
		for tword in twords:
			tagged_text += tword
			if word_index < len (self.words) and tword == self.words [word_index].word:
				tagged_text += "_" + self.words [word_index].pos_tag
				word_index += 1
			tagged_text += " "
		tagged_text = tagged_text.strip ()
		
		return tagged_text
		
class Section:

	randomized_section_list = []
	sections_to_be_used = []
	section_number = 0

	def __init__ (self, title, body):

		self.parent = None
		self.children = []
		self.descendants = None
		self.all_descendants = None
		self.title = Sentence (title)

		self.body = [Sentence (x)\
					  for x in utils.split_into_sentences (body, config.SENTENCE_DELIMETERS [config.DATASET_NAME])]
		self.unique_word_objs = {}
		
		self.first_occurrence_in_corpus_by_section = {}
		self.first_occurrence_by_sentence = {}
		self.first_occurrence_by_word = {}

		self.__stem_to_count = {}
		self.__word_to_count = {}

		self.__stem_to_first_occurrence_by_word = {}
		self.__stem_to_first_occurrence_by_sentence = {}
		self.__stem_to_sentence_indices = {}
		self.__stem_to_first_section = {}
		
		self.__np_to_count = {}
		self.__np_to_corpus_count = {}

		for sentence_position, sentence in enumerate (self.body):

			for word_position, word in enumerate (sentence.words):

				if not self.__stem_to_first_occurrence_by_word.has_key (word.stem):
					self.__stem_to_first_occurrence_by_word [word.stem] = word_position
					
				if not self.__stem_to_first_occurrence_by_sentence.has_key (word.stem):
					self.__stem_to_first_occurrence_by_sentence [word.stem] = 1.0 * sentence_position
				
				self.__stem_to_count [word.stem] = self.__stem_to_count.get (word.stem, 0) + 1
				self.__word_to_count [word.word] = self.__word_to_count.get (word.word, 0) + 1

				if not self.__stem_to_sentence_indices.has_key (word.stem):
					self.__stem_to_sentence_indices [word.stem] = set ()
				self.__stem_to_sentence_indices [word.stem].add (sentence_position)
		
		self.total_unigrams = sum (self.__word_to_count.values ())
		

	# ----------------------------------------------------------------------------
	def set_sections_to_be_used (self, sections):
		Section.sections_to_be_used = sections
		print Section.sections_to_be_used


	# ----------------------------------------------------------------------------
	def add_child (self, child_section):

		child_section.parent = self
		self.children.append (child_section)


	# ----------------------------------------------------------------------------
	def body_words (self):
		for sentence in self.body:
			for word in sentence.words:
				yield word

	# ----------------------------------------------------------------------------
	def create_unique_word_list (self):
		unique_words = {}
		for sentence in self.body:
			for word in sentence.words:
				unique_words [word.stem + '|' + str (word.pos_tag)] = word
		self.unique_word_objs = unique_words.values ()


	# ----------------------------------------------------------------------------
	def get_all_descendants (self):

		# If this is the root section, return the precomputed list of descendants
		if self.is_root () and self.all_descendants != None:
			return self.all_descendants

		# Compute the descendants
		descendants = [self]
		descendants.extend (reduce (lambda x, y: x + y,  \
									map (lambda child: \
												child.get_all_descendants (), \
												self.children), \
									[]))

		# Cache if this is the root section
		if self.is_root ():
			self.all_descendants = descendants
			self.__build_first_section_with_stem_map ()

		return descendants


	# ----------------------------------------------------------------------------
	def get_descendants_old (self, depth):

		# If this is the root section, return the precomputed list of descendants
		if self.is_root () and self.descendants != None:
			return self.descendants

		# Compute the descendants
		descendants = [self]

		descendants.extend (reduce (lambda x, y: x + y,  \
									map (lambda child: \
												child.get_descendants_old (depth - 1), \
												self.children), \
									[]))

		# Cache if this is the root section
		if self.is_root ():
			print "----------------------------------"
			print len (descendants)
			print "----------------------------------"
			self.descendants = descendants

		return descendants


	# ----------------------------------------------------------------------------
	def get_descendants (self, depth):

		if False == self.is_root ():
			return None

		# If this is the root section, return the precomputed list of descendants
		if self.descendants != None:
			return self.descendants

		# If this is the root section, find out at what depth we actually start	
		# using sections for training & testing...								
		#if self.is_root ():

		section_list = []
		for sec_num in Section.sections_to_be_used:
			sec = self.get_section_by_number (0, sec_num)
			if sec == None:
				print "[ERROR]  section " + str (sec_num) + " not found!"
			print str (sec_num) + '  ' + sec.title.text
			section_list.append (sec)

		Section.section_number = -1
		depth = config.SUBTREE_DEPTH
		print "----------------------------------"
		print " finding section list " + str (depth)
		print "----------------------------------"

		descendants = []

		descendants.extend (reduce (lambda x, y: x + y,  \
									map (lambda child: \
												child.get_descendants_old (depth - 1), \
												section_list), \
									[]))

		# Cache if this is the root section
		# if self.is_root ():
		print "----------------------------------"
		print len (descendants)
		print "----------------------------------"
		self.descendants = descendants

		return descendants


	# ----------------------------------------------------------------------------
	def get_section_by_number (self, depth, section_to_find):

		if self.is_root ():
			Section.section_number = -1
			depth = config.SUBTREE_DEPTH

		# Compute the descendants
		if depth == 0:
			Section.section_number = Section.section_number + 1
			if Section.section_number == section_to_find:
				return self
		
		if depth > 0:
			for child in self.children:
				sec = child.get_section_by_number (depth - 1, section_to_find)
				if (sec != None):
					return sec

		return None



	# ----------------------------------------------------------------------------
	def capitalize_title (self, title_words):

		# Build capitalization map
		word_to_capital = {}
		for sentence in self.body: 
			words = sentence.text.split (" ")

			for i, word in enumerate (words):
				if i !=0 or (i == 0 and word.isupper ()): 
					lower_word = word.lower ()
					if not word_to_capital.has_key (lower_word):
						word_to_capital [lower_word] = {}
					word_to_capital [lower_word] [word] = word_to_capital [lower_word].get (word, 0) + 1

		# Find max case for each word
		for word, cased_word_to_count in word_to_capital.items ():
			max_case = None
			max_case_count = -1
			for cased_word, count in cased_word_to_count.items ():
				if max_case_count <= count:
					max_case_count = count
					max_case = cased_word
			word_to_capital [word] = max_case		

		# Case the title
		capitalized_title = []
		first_word = True
		for word in title_words:
			if word == START_WORD:
				capitalized_title.append (word.word)
				continue
			if first_word:
				capitalized_word = word_to_capital.get (word.word, word.word)
				if capitalized_word.islower ():
					capitalized_title.append (capitalized_word.capitalize ())
				else:
					capitalized_title.append (capitalized_word)
				first_word = False
			else:
				capitalized_title.append (word_to_capital.get (word.word, word.word))
		
		return capitalized_title


	def is_root (self): return self.parent == None


	def __build_first_section_with_stem_map (self):
		if not self.is_root ():
			raise Exception ('this should only be called on the root node')
		
		for sec_num, sec in enumerate (self.all_descendants):
			for stem in sec.__stem_to_count.keys ():
				self.__stem_to_first_section [stem] = self.__stem_to_first_section.get (stem, sec_num)


	def build_np_counts (self):
		if len (self.__np_to_corpus_count.keys ()) > 0:
			return
		if not self.is_root ():
			raise Exception ('this should only be called on the root node')

		for sec in self.get_all_descendants ():
			for sentence in sec.body:
				for np in utils.extract_nps (sentence.text, sentence.words):
					np_string = utils.words_to_string (np)
					sec.__np_to_count [np_string] = sec.__np_to_count.get (np_string, 0) + 1 
			
		for sec in self.get_all_descendants ():
			for (np, count) in sec.__np_to_count.items ():
				self.__np_to_corpus_count [np] = self.__np_to_corpus_count.get (np, 0) + count


	############################
	def get_depth (self):
		depth = 0
		p = self.parent
		while p != None:
			depth += 1
			p = p.parent
		return depth

	def has_unigram (self, unigram, use_stem):
		return (use_stem and (unigram in self.__stem_to_count))\
			   or ((not use_stem) and (unigram in self.__word_to_count))

	def get_unigram_occurrence (self, unigram, use_stem):
		if use_stem: return self.__stem_to_count.get (unigram, 0)
		else: return self.__word_to_count.get (unigram, 0)

	def get_num_descendants_with_unigram (self, unigram, use_stem):
		return reduce (lambda x, y: x + y.has_unigram (unigram, use_stem), self.get_all_descendants (), 0)

	def get_total_unigram_occurrences (self):
		return self.total_unigrams

	def get_first_occurrence_by_word (self, stem):
		return self.__stem_to_first_occurrence_by_word [stem]

	def get_first_occurrence_by_sentence (self, stem):
		return self.__stem_to_first_occurrence_by_sentence [stem]

	def get_first_occurrence_by_section (self, stem):
		assert (self.is_root ())
		# Note: If stem is not found in corpus we return the last section
		return self.__stem_to_first_section.get (stem, len (self.get_all_descendants ()))

	def get_np_count (self, np_string):
		return self.__np_to_count.get (np_string, 0) 

	def get_body_nps (self):
		return self.__np_to_count.keys ()

	def get_corpus_np_count (self, np_string):
		if not self.is_root ():
			raise Exception ('this should only be called on the root node')
		return self.__np_to_corpus_count.get (np_string, 0) 

	def all_words_occur_in_same_sentence (self, history, word, stopwords):
		sentence_indices = self.__stem_to_sentence_indices [word.stem]
		nonstopwords_encountered = not (word.stem in stopwords)
		for history_word in history:
			if history_word == START_WORD or history_word == END_WORD: continue
			if history_word in stopwords: continue
			if not self.__stem_to_sentence_indices.has_key (history_word.stem): return 0.0

			if not nonstopwords_encountered:
				sentence_indices = self.__stem_to_sentence_indices [history_word.stem]
			else:
				sentence_indices = sentence_indices & self.__stem_to_sentence_indices [history_word.stem]
			nonstopwords_encountered = True
			if (len (sentence_indices) == 0): return 0.0
	
		if len (sentence_indices) > 0:
			return 1.0
		else:
			return 0.0


	def previous_section_has_stem (self, stem):

		if self.parent == None:
			return 0
		if self.parent.children [0] == self:
			return 0
		previous_section = self.parent.children [self.parent.children.index (self) - 1]
		if previous_section.__stem_to_count.get (stem, 0) != 0:
			return 1
		else:
			return 0

	def next_section_has_stem (self, stem):

		if self.parent == None:
			return 0
		if self.parent.children [-1] == self:
			return 0
		next_section = self.parent.children [self.parent.children.index (self) + 1]
		if next_section.__stem_to_count.get (stem, 0) != 0:
			return 1
		else:
			return 0

	def has_bigram (self, previous_word, current_word, stopwords):
		if previous_word.word.lower () in stopwords or current_word.word.lower () in stopwords:
			return 100
		if previous_word.pos_tag in non_nouns or current_word.pos_tag in non_nouns:
			return 100
		bigram = previous_word.word.lower () + ' ' + current_word.word.lower ()
		occurrences = 0
		for sentence in self.body:
			occurrences += sentence.text.lower ().count (bigram)
		return occurrences




