import config, utils, section
import os

class LanguageModel:

    def __init__ (self):

        self.order = None
        self.ngram_to_score = {}
        self.ngram_to_backoff = {}


    def train (self, sentence_text_generator, order):

        arpa_output_path = config.LANGUAGE_MODEL_TEMP_DIR + "maxent_lm_arpa_file.text"
        lm_input_file = config.LANGUAGE_MODEL_TEMP_DIR + "maxent_lm_imput.text"
        
        # Write to file
        f = open (lm_input_file , 'w')
        for sentence_text in sentence_text_generator:
            sentence_text = sentence_text.replace ('<s>', '<S>')
            f.write (sentence_text + "\n")
        f.close ()

        # Train with SRILM
        lp_pipe = os.popen (config.LANGUAGE_MODEL_SRILM_PATH 
							+ "ngram-count -order " + str (order) 
							+ " -text " + lm_input_file 
							+ " -lm " + arpa_output_path)
        lp_pipe.close ()
        #os.unlink (lm_input_file)

        self.__load_arpa_file (arpa_output_path)
        #os.unlink (arpa_output_path)
        

    def __load_arpa_file (self, arpa_output_path):

        self.order = 0

        f = open (arpa_output_path , 'r')
        lines = f.readlines ()
        f.close ()

        for line in lines:
            line = line.replace ('<s>', section.START_WORD.word)
            line = line.replace ('</s>', section.END_WORD.word)
            
            line_parts = line.strip ().split ("\t")

            # Skip lines that do not have ngrams and weights
            if len (line_parts) < 2:
                continue

            # Store ngram probability
            prob = float (line_parts[0])
            ngram = line_parts[1].strip ()
            self.ngram_to_score[ ngram ] = prob

            # Calculate order
            self.order = max ([ len (ngram.split (" ")), self.order])	# <<<<<<<<<<<<
                
            # Store back off weight
            if len (line_parts) == 3:
                backoff = float (line_parts[2])
                self.ngram_to_backoff[ ngram ] = backoff


    # TODO: Enable higher orders            
    def get (self, ngrams ):

        if len (ngrams) > self.order:
            raise Exception ("Supplied ngram size " + str (len (ngrams)) + \
                             " does not match the language model order " + str (self.order))

        last_start_index = utils.find_last_start_token (ngrams, section.START_WORD)
        if last_start_index > 0:
            ngrams = ngrams[last_start_index:]

        probability = self.get_ngram_prob ( ngrams )

        return probability

    def get_ngram_prob (self, ngram ):

        def ngram_to_string( ngram ):
            return ' '.join ([str (x) for x in ngram])
        
        unk_prob = -100.0
        unk_uncertainty = -100.0
        
        # Check the order
        if self.order != 3:
            raise Exception( "Currently, this only works for trigram lms" )

        # 0-GRAM: Error case
        if len(ngram) == 0:
            return ( unk_prob, unk_uncertainty )

        # 1-GRAM: Return the score P(A)
        if len(ngram) == 1:
            unigram = ngram_to_string( ngram )
            if self.ngram_to_score.has_key( unigram ):
                prob = self.ngram_to_score[ unigram ]
                return ( prob, 0 )
            else:
                return ( unk_prob, unk_uncertainty )

        # 2-GRAM: Return score, interpolate if not found P(B|A)
        if len(ngram) == 2:

            # If the bigram exists: P(B|A)
            bigram_string = ngram_to_string( ngram ) 
            if self.ngram_to_score.has_key( bigram_string ):
                prob = self.ngram_to_score[ bigram_string ]
                return ( prob, 0 )

            # Interpolate P(B) + B(A)
            # TODO: Recursive call
            ( prob, uc ) = self.get_ngram_prob( [ ngram[1] ] )
            backoff = self.ngram_to_backoff.get( ngram[0], 0.0 )
            return ( prob + backoff, uc - 1 )
        
        # 3-GRAM: Return score, interpolate if not found P(C|B,A)
        if len(ngram) == 3:

            # If trigram exists: P(C|B,A)
            trigram_string = ngram_to_string( ngram )
            if self.ngram_to_score.has_key( trigram_string ):
                prob = self.ngram_to_score[ trigram_string ]
                return ( prob, 0 )

            # Interpolate P(C|B) + B(B|A)
            ( prob, uc ) = self.get_ngram_prob( ngram[-2:] )
            prob += self.ngram_to_backoff.get( ngram_to_string( ngram[:2] ), 0.0 )
            return ( prob, uc - 1 )

        raise Exception("Unknown code path, prior return expected")

    '''
    def get_ngram_prob_broken (self, ngram ):

        probability = self.ngram_to_score.get (' '.join ([str (x) for x in ngram]), None)
        if probability != None:
            return ( 1.0 * probability, 0.0 )

        # if this is the first word or the last word, don\'t smooth the language model scores...
        #if len (ngram) > 1:
        #    if ngram [-2] == (section.START_WORD.word + '_' + section.START_WORD.word) and ngram [-1] != (section.START_WORD.word + '_' + section.START_WORD.word):
        #        return ( -100, -100 )
        #if len (ngram) > 0:
        #    if ngram [-1] == (section.END_WORD.word + '_' + section.END_WORD.word):
        #        return ( -100, -100 )
        #elif not smoothing:
        #    return config.LANGUAGE_MODEL_NO_SMOOTHING_DEFAULT_LOG_PROB

        if len (ngram) < 1:
            return ( -100, -100 )

        ( probability, uncertainty ) = self.get_ngram_prob (ngram[1:] )
        backoff = self.ngram_to_backoff.get (ngram[0], 0.0)
        print str(ngram[1:]) + " " + str(probability) + str(backoff)

        return ( 1.0 * probability + backoff, uncertainty - 1 ) 
    '''
    
    def get_uncertainty_score( self, ngrams ):
        uc = 0
        for i in range( len(ngrams) - self.order ):
            uc += self.get_ngram_prob( ngrams[i:i+self.order] )[1]
        return uc
