import section, config, utils 
import os, math

class Model:

    def __init__( self, root_section, window_size, tag_lm, lexical_lm, stopwords_set, stem_m ):
        self.feature_vectors = []
        self.root_section = root_section
        self.window_size = window_size
        self.tag_lm = tag_lm
        self.lexical_lm = lexical_lm
        self.model_path = config.PICKLE_PATH + "model.zlm"
        self.stopwords_set = stopwords_set
        self.stem_model = stem_m
        
    def add_training_features_section( self, sec, previous_sec ):
        #utils.p( "* Adding features for section: " + sec.title.text + str (len (sec.unique_word_objs)))
        utils.p( "* Adding features for section: " + ' '.join( [ x.word + "/" + x.pos_tag for x in sec.title.words ] ) )
        
        # Initial start word padding 
        history_words = map( lambda x: section.START_WORD, range( self.window_size - 1 ))
        full_history = map( lambda x: section.START_WORD, range( self.window_size - 1 ))

        title_words = sec.title.words
        title_words.append( section.END_WORD )
        
        for title_word in title_words:
            #-----------------------------------------------------------
            # for loop for use with de-stemming...                      
            # for word in self.stem_model.get_words_by_section( sec ):  
            #-----------------------------------------------------------
            for word in sec.unique_word_objs:
                in_title = (word.word == title_word.word) and (word.pos_tag == title_word.pos_tag)
                self.__add_event( FeatureVector(full_history, history_words, word, in_title, sec, previous_sec, self ) )

            history_words.append(title_word)
            history_words.pop(0)
            full_history.append(title_word)           
        self.__add_event( FeatureVector(full_history, history_words, section.END_WORD, True, sec, previous_sec, self ) )


    def __add_event( self, feature_vector ):
        self.feature_vectors.append( feature_vector )


    def train( self ):
        utils.p( " * Training..." )
        self.train_zhangle ()
        

    def train_zhangle (self):
        feature_path = config.MAXENT_TEMP_DIR + "maxent_train.fv"
        
        # Generate feature vector file
        f = open ( feature_path , 'w' )
        for fv in self.feature_vectors:
            f.write( fv.render_zhangle() + "\n" )
        f.close ()

        maxent_commandline = config.MAXENT_ZHANGLE_EXEC_PATH + " " + feature_path + " -b -m " + self.model_path + " -i 14 -v"
        self.run_training_command (maxent_commandline, feature_path)


    def train_yasmet (self):
        feature_path = config.MAXENT_TEMP_DIR + "maxent_train.fv"
        
        # Generate feature vector file
        f = open ( feature_path , 'w' )
        f.write ("2\n")
        f.write ( '\n'.join( [ x.render_yasmet() for x in self.feature_vectors ] ) )
        f.close ()

        maxent_commandline = "cat " + feature_path + " | " + config.MAXENT_YASMET_EXEC_PATH + \
                             " -deltaPP 0.0 -iter 100 > " + self.model_path
        self.run_training_command (maxent_commandline, feature_path)


    def run_training_command (self, commandline, feature_path):
        # Train model
        if os.access (feature_path + '.1', os.W_OK):
            os.unlink (feature_path + '.1')

        if os.access (self.model_path, os.F_OK):
            os.unlink (self.model_path)
            
        utils.p ('maxent command line : ' + commandline)
        #p = os.popen (commandline)
        #p.close()
        if os.access (feature_path + '.1', os.W_OK)  or not os.access (feature_path + '.1', os.F_OK):
            os.rename (feature_path, feature_path + '.1')

        # Remove feature vectors
        self.feature_vectors = None


    # eval function for yasmet maxent package
#    def evaluate( self, feature_vectors ):
#        output_path = os.tempnam (config.MAXENT_TEMP_DIR, 'mxt.')
#
#        # Create test file
#        test_path = os.tempnam (config.MAXENT_TEMP_DIR, 'mxfv.')
#
#        f = open( test_path, 'w' )
#        f.write('2\n')
#        f.write( '\n'.join( [ x.render() for x in feature_vectors ] ) )
#        f.close()
#
#        # Test
#        p = os.popen( config.MAXENT_EXEC_PATH + " -p -b -m " + self.model_path\
#                        + " --detail --output=" + output_path + " " + test_path)
#        p.close()
#        os.unlink (test_path)
#
#        # Parse output
#        f = open(output_path, 'r')
#        lines =  f.readlines()
#        f.close()
#        os.unlink (output_path)
#
#        output_probs = []
#        for line in lines:
#            [class1, prob1, class2, prob2] = line.strip().split(" ")
#            if class1 == "t": output_probs.append(float(prob1))
#            if class2 == "t": output_probs.append(float(prob2))
#
#        assert( len(feature_vectors) == len(output_probs) )
#
#        return output_probs

        

    # eval function for zhang le's maxent package
    def evaluate( self, feature_vectors ):
        output_path = os.tempnam (config.MAXENT_TEMP_DIR, 'mxt.')

        # Create test file
        test_path = os.tempnam (config.MAXENT_TEMP_DIR, 'mxfv.')

        f = open( test_path, 'w' )
        f.write( '\n'.join( [ x.render_zhangle() for x in feature_vectors ] ) )
        f.close()

        # Test
        p = os.popen( config.MAXENT_ZHANGLE_EXEC_PATH + " -p -b -m " + self.model_path\
                      + " --detail --output=" + output_path + " " + test_path)
        p.close()
        os.unlink (test_path)

        # Parse output
        f = open(output_path, 'r')
        lines =  f.readlines()
        f.close()
        os.unlink (output_path)

        output_probs = []
        for line in lines:
            [class1, prob1, class2, prob2] = line.strip().split("\t")
            if class1 == "t": output_probs.append(float(prob1))
            if class2 == "t": output_probs.append(float(prob2))

        assert( len(feature_vectors) == len(output_probs) )

        return output_probs
        
class FeatureVector:


    def __init__( self, full_history, history_words, word, in_title, sec, previous_sec, model ):

        

        root_sec = model.root_section
        tag_lm = model.tag_lm
        lexical_lm = model.lexical_lm

        self.feature_values = []
        self.in_title = in_title

        all_sections = root_sec.get_descendants ()
        section_count = len (all_sections)
        section_number = all_sections.index(sec)

        # PRINT WORDS TO FEATURE FILE FOR DEBUGGING.
        #self.__add_value ("words", str( [ x.word for x in (history_words + [word]) ])  )

        # ORDERING FEATURES ################
        
        # Feature: Title Tag LM
        # TODO: Extend past bigram
        ngram_tags = map (lambda x: x.pos_tag, history_words)
        ngram_tags.append (word.pos_tag)
        ( feat_taglm_score, feat_taglm_uc ) = tag_lm.get (ngram_tags)
        feat_taglm_uc += tag_lm.get_uncertainty_score( map (lambda x: x.pos_tag, full_history ))
        self.__add_value( "taglm", -1.0 * feat_taglm_score )
        self.__add_value( "taglm_uncertainty", -1.0 * feat_taglm_uc)
        
        # Feature: Lexical LM
        # TODO: Extend past bigram
        if not (section.START_WORD == history_words[-1] or section.END_WORD == word):
            ngram_words = map (lambda x: x.word + '_' + x.pos_tag, history_words)
            ngram_words.append (word.word + '_' + word.pos_tag)
            ( feat_lexlm, feat_lexlm_uc) = lexical_lm.get( ngram_words )
            feat_lexlm_uc += lexical_lm.get_uncertainty_score( map (lambda x: x.word + '_' + x.pos_tag, full_history ))
            self.__add_value( "lexical_lm", -1.0 * feat_lexlm )
            self.__add_value( "lexical_lm_uncertainty", -1.0 * feat_lexlm_uc )

        # SPECIAL FEATURES #################
        self.__add_value( "is_end_word", (word == section.END_WORD) and "1" or "0" )

        # Do not calculate the below features for the end word
        if word == section.END_WORD: return

        # SELECTION FEATURES ###############

        # Feature: POS Tag
        self.__add_value( "pos_tag_" + str(word.pos_tag) , 1 )
        
        # Feature: Is word a stopword
        self.__add_value( "is_stopword", word in model.stopwords_set and 1 or 0 )
        
        # Feature: previous/next/parent section has stem
        self.__add_value ("previous_section_has_stem", sec.previous_section_has_stem (word.stem))
        self.__add_value ("next_section_has_stem", sec.next_section_has_stem (word.stem))
        self.__add_value ("parent_section_has_stem", sec.parent_section_has_stem (word.stem))

        # Feature: first occurrence in corpus by section      
        feat_first_occurrence_in_corpus_by_section = 1.0 * (section_number - root_sec.get_first_occurrence_by_section (word.stem))
        self.__add_value ( "first_occurrence_in_corpus_by_section", feat_first_occurrence_in_corpus_by_section)

        # Feature: first occurrence in section by sentence, without normalization
        feat_first_occurrence_by_sentence_not_normalized = sec.get_first_occurrence_by_sentence( word.stem )
        if feat_first_occurrence_by_sentence_not_normalized > 20: feat_first_occurrence_by_sentence_not_normalized = 20
        self.__add_value ( "first_occurrence_by_sentence", feat_first_occurrence_by_sentence_not_normalized )
        
        # Feature: first occurrence in section by word       
        sec.first_occurrence_by_word[word.stem] = sec.first_occurrence_by_word.get (word.stem, \
                                                                                1.0 * sec.get_first_occurrence_by_word( word.stem ) / sec.get_total_unigram_occurrences())
        self.__add_value ( "first_occurrence_by_word", sec.first_occurrence_by_word[word.stem])

        # Feature TF IDF
        feat_tf = 1.0 * sec.get_unigram_occurrence( word.stem, True ) / sec.get_total_unigram_occurrences()
        feat_df = 1.0 * root_sec.get_num_descendants_with_unigram( word.stem, True) / section_count
        self.__add_value( "tf-idf", feat_tf * ( -1.0 / math.log( feat_df ) ) )
    
        # Differential TF-IDF  (does not seem to change anything...)
        #if previous_sec != None:
        #    previous_sec_total_unigrams = previous_sec.get_total_unigram_occurrences()
        #    if previous_sec_total_unigrams > 0:
        #        feat_prevsec_tf = 1.0 * previous_sec.get_unigram_occurrence( word.stem, True ) / previous_sec_total_unigrams
        #        feat_diff_tf = feat_tf - feat_prevsec_tf
        #        if feat_diff_tf < 0:
        #            feat_diff_tf = 0
        #        self.__add_value( "diff_tf-idf", feat_diff_tf)

        # Feature bigram count in section
        #if history_words [-1] != None:
        #    feat_has_bigram = 1.0 * sec.has_bigram (history_words [-1], word, model.stopwords_set)
        #    self.__add_value ("section_bigram_count", feat_has_bigram)

        # Feature NP count in section
        all_words = full_history + [word]
        nps = utils.extract_nps( utils.words_to_string( all_words ), all_words )
        
        if len(nps) > 0 and nps[-1][-1] == word:
            section_np_count = sec.get_np_count( utils.words_to_string( nps[-1] ) ) 
            corpus_np_count = root_sec.get_corpus_np_count( utils.words_to_string( nps[-1] ) ) 
            if corpus_np_count > 0:
                utils.p (str (map (lambda x:x.word, nps[-1])) + ' -- ' + str (1.0 * (section_np_count + 1)/corpus_np_count))
                self.__add_value ("section_np_ratio", 1.0 * (section_np_count + 1)/corpus_np_count)
            else:
                self.__add_value ("section_np_ratio", 0)
            

    def __add_value( self, name, value ):
        self.feature_values.append( (name, value) )
    

    def render_zhangle ( self ):
        fv_string = (self.in_title and "t" or "f") + " "
        fv_string += ' '.join( [ str(feature) + ":" + str(value) for feature, value in self.feature_values ] )
        return fv_string



    def render_yasmet ( self ):
        fv_string = (self.in_title and "1" or "0") + " @ "

        fv_string += "@ " + (self.in_title and "1" or "0") + " "
        for feature, value in self.feature_values:
            fv_string += str(feature) + "_t " + str(value) + " "
        fv_string += "# "

        fv_string += "@ " + (self.in_title and "0" or "1") + " "
        for feature, value in self.feature_values:
            fv_string += str(feature) + "_f " + str(value) + " "
        fv_string += "#"

        return fv_string
