import config, section
from xml.dom import minidom, Node

def read_corpus():
    
    if config.DATASET_NAME  == config.DATASET_ALGORITHMS:
        xmldoc = minidom.parse( config.DATAFILE_ALGORITHMS )        
        return read_corpus_algorithms( xmldoc )
    if config.DATASET_NAME  == config.DATASET_PHYSICS_LECTURES:
        xmldoc = minidom.parse( config.DATAFILE_PHYSICS_LECTURES )        
        return read_corpus_physics_lectures( xmldoc )
    if config.DATASET_NAME  == config.DATASET_ALGORITHMS_RANDOMIZED:
        xmldoc = minidom.parse( config.DATAFILE_ALGORITHMS_RANDOMIZED )        
        return read_corpus_algorithms( xmldoc )
    if config.DATASET_NAME  == config.DATASET_CIVIL:
        xmldoc = minidom.parse( config.DATAFILE_CIVIL )
        return read_corpus_civil( xmldoc )
    if config.DATASET_NAME  == config.DATASET_PHYSICS:
        xmldoc = minidom.parse( config.DATAFILE_PHYSICS )
        return read_corpus_physics( xmldoc )
    if config.DATASET_NAME  == config.DATASET_MERCK:
        xmldoc = minidom.parse( config.DATAFILE_MERCK )
        return read_corpus_merck( xmldoc )        
    else:
        raise Exception( "No reader for " + config.DATASET_NAME + " found." )

def read_corpus_algorithms( xmldoc ):
    
    root_section = section.Section("","")
    for child_node in xmldoc.childNodes:
        for grandchild_node in child_node.childNodes:
            root_section.add_child( make_section_algorithms( grandchild_node ) )

    return root_section

def read_corpus_physics_lectures( xmldoc ):
    
    root_section = section.Section("","")
    for child_node in xmldoc.childNodes:
        for grandchild_node in child_node.childNodes:
			root_section.add_child( make_section_algorithms( grandchild_node ) )

    return root_section

def read_corpus_physics( xmldoc ):
    
    root_section = section.Section("","")
    for child_node in xmldoc.childNodes:
        for grandchild_node in child_node.childNodes:
            root_section.add_child( make_section_physics( grandchild_node ) )

    return root_section

def read_corpus_civil( xmldoc ):
    
    root_section = section.Section("","")
    for child_node in xmldoc.childNodes:
        root_section.add_child( make_section_civil(child_node ) )
            
    return root_section

def read_corpus_merck( xmldoc ):
    
    root_section = section.Section("","")
    for child_node in xmldoc.childNodes:
        for grandchild_node in child_node.childNodes:
            root_section.add_child( make_section_merck(grandchild_node ) )
            
    return root_section

def make_section_physics( section_node ):

    text = None
    title = None
    child_sections = []
    
    for child_node in section_node.childNodes:
        
        # Add child sections
        if child_node.tagName == "section":
            child_sections.append( make_section_physics(child_node) )
            
        # Set title
        if child_node.tagName == "title":
            title = child_node.childNodes[0].nodeValue
            
        # Set text
        if child_node.tagName == "body":
            text = child_node.childNodes[0].nodeValue

    # TODO: Fix this
    text = text.replace("#FIGURE#", "<expr>")
    text = text.replace("&nbsp;", " ")
    
    s = section.Section( title, text )
    for child_section in child_sections:
        s.add_child( child_section )

    return s

def make_section_merck( section_node ):

    text = None
    title = None
    child_sections = []
    
    for child_node in section_node.childNodes:

        if child_node.nodeType != Node.ELEMENT_NODE: continue
        
        # Add child sections
        if child_node.tagName == "section":
            child_sections.append( make_section_civil(child_node) )
            
        # Set title
        if child_node.tagName == "title":
            title = child_node.childNodes[0].nodeValue
            
        # Set text
        if child_node.tagName == "body":
            text = len( child_node.childNodes ) > 0 and child_node.childNodes[0].nodeValue or None

    title = title.encode('ascii', 'replace').replace("\n", " ")
    text = text.encode('ascii', 'replace').replace("\n", " ")
    s = section.Section( title, text )

    for child_section in child_sections:
        s.add_child( child_section )

    return s

def make_section_civil( section_node ):

    text = None
    title = None
    child_sections = []
    
    for child_node in section_node.childNodes:

        if child_node.nodeType != Node.ELEMENT_NODE: continue
        
        # Add child sections
        if child_node.tagName == "part":
            child_sections.append( make_section_civil(child_node) )
            
        # Set title
        if child_node.tagName == "title":
            title = child_node.childNodes[0].nodeValue
            
        # Set text
        if child_node.tagName == "text":
            text = len( child_node.childNodes ) > 0 and child_node.childNodes[0].nodeValue or None

    text = text != None and text.replace("<br>", "") or ""
    s = section.Section( title, text )

    for child_section in child_sections:
        s.add_child( child_section )

    return s
    
def make_section_algorithms( section_node ):

    text = None
    title = None
    child_sections = []
    
    for child_node in section_node.childNodes:
        
        # Add child sections
        if child_node.tagName == "section":
            child_sections.append( make_section_algorithms(child_node) )
            
        # Set title
        if child_node.tagName == "title":
            title = ( len( child_node.childNodes ) > 0 ) and child_node.childNodes[0].nodeValue or ""
            
        # Set text
        if child_node.tagName == "body":
            text = ( len( child_node.childNodes ) > 0 ) and child_node.childNodes[0].nodeValue or ""

    #title = title.replace("<num>", "9999")
    text = text.replace("<num>", "9999")
    s = section.Section( title, text )
    for child_section in child_sections:
        s.add_child( child_section )

    return s
