import config, utils
import os

DOCUMENT_START_TAG = "<DOCUMENT>"
DOCUMENT_END_TAG = "</DOCUMENT>"

OUTPUT_DIRECTORY = config.TAGGER_OUTPUT_DIR + config.DATASET_NAME

def tag_corpus( root_section ):

    all_sections = root_section.get_descendants()

    # Make tagger output directory
    utils.p ('* Making tagged output directory')
    tag_output_dir = OUTPUT_DIRECTORY
    if not os.access (tag_output_dir, os.F_OK):
        os.mkdir( tag_output_dir )

    # Make input file, one sentence per line
    utils.p ('* Making mxpost input file')
    tagger_input_file = config.TAGGER_TEMP_DIR + config.DATASET_NAME + "_maxent_tagger_input.txt"
    f = open(tagger_input_file, 'w')

    for sec in all_sections:
        write_tag_input_by_section( sec, f )
        
    f.close()

    # Run the tagger
    utils.p ('* Calling mxpost')
    lp_pipe = os.popen(config.TAGGER_PATH + " < " + tagger_input_file)
    tagger_output = lp_pipe.read()
    lp_pipe.close()
    #utils.p ('* Removing mxpost input file')
    #os.unlink (tagger_input_file)

    # Write tagger output to file
    utils.p ('* writting mxpost output to storage') 
    f = open( OUTPUT_DIRECTORY + '/' + config.DATASET_NAME + '_output.tag', 'w')
    f.write( tagger_output )
    f.close()

    utils.p ('* Parsing mxpost output')
    __parse_tagged_output( tagger_output, all_sections)

def write_tag_input_by_section( section, f ):
    utils.p( "* Tagging section: " + section.title.text + "..." )    

    f.write(DOCUMENT_START_TAG+".\n")
    f.write( section.title.text + "\n")
    for sentence in section.body:
        f.write(sentence.text + "\n")
    f.write(DOCUMENT_END_TAG+".\n")

def tag_from_file( root_section ):
    # Read tagger output
    file_name = OUTPUT_DIRECTORY + '/' + config.DATASET_NAME + '_output.tag'
    f = open(file_name, 'r')
    tagger_output = f.read()
    f.close()
   
    __parse_tagged_output( tagger_output, root_section.get_descendants() )

def __parse_tagged_output(tagger_output, all_sections):
    read_section_output = False
    section_output = ""
    section_number = 0
    
    for line in tagger_output.split("\n"):
        if line.startswith(DOCUMENT_START_TAG):
            read_section_output = True
        elif line.startswith(DOCUMENT_END_TAG):
            read_section_output = False
            __parse_tagged_section_output( section_output, all_sections[section_number ] )
            section_number += 1
            section_output = ""
        elif read_section_output:
            section_output += line + "\n"
    
def __parse_tagged_section_output(tagger_output, section):
    # Parse output
    [title_tagged, body_tagged] = tagger_output.split("\n", 1)
    __set_word_tags(title_tagged, section.title)
    body_tagged_lines = body_tagged.split("\n")
    for i in range( len( section.body ) ):
        __set_word_tags(body_tagged_lines[i], section.body[i] )
    section.create_unique_word_list ()

def __set_word_tags( tagged_text, sentence ):

    sentence_word_index = 0
    for word_tag in tagged_text.split(" "):

        if sentence_word_index >= len( sentence.words ): return

        wt_parts = word_tag.split("_")
        if len( wt_parts ) != 2: continue
        [word, tag] = wt_parts
        word = word.lower()
        
        if word == sentence.words[ sentence_word_index ].word:


            if config.DATASET_NAME == config.DATASET_ALGORITHMS:
                #--------------------------------------------------
                # the following lines are to correct tagging mistakes on the algorithms corpus by mxpost
                if (word == 'shows' or word == 'computes' or \
                    word == 'determines' or word == 'lies' or \
                    word == 'straddles' or word == 'varies' or \
                    word == 'approaches' or word == 'oscillates' or \
                    word == 'investigates') and tag == 'NNS':
                    tag = 'VBZ'
                elif word == 'show' and (tag == 'NN' or tag == 'NNP'):
                    tag = 'VB'
                elif word == 'in' and tag == 'NNP':
                    tag = 'IN'
                elif word == 'that' and tag == 'IN':
                    tag = 'WDT'
                elif word == '9999':
                    tag = 'CD'
                #--------------------------------------------------

            sentence.words[ sentence_word_index ].pos_tag = tag
            sentence_word_index += 1

def tag_titles_from_text( root_section ):
    # Build word to tag counts from body text
    word_to_tag = {}

    for sec in root_section.get_descendants():
        for word in sec.body_words():
            if not word_to_tag.has_key( word.word ):
                word_to_tag[ word.word ] = {}
            word_to_tag[ word.word ][ word.pos_tag ] = word_to_tag[ word.word].get( word.pos_tag, 0 ) + 1

    # Get majority POS tag for each word
    for word, pos_tag_counts in word_to_tag.items():
        max_tag = None
        max_tag_count = -1
        for tag, count in pos_tag_counts.items():
            if max_tag_count <= count:
                max_tag_count = count
                max_tag = tag
        word_to_tag[ word ] = max_tag

    # Tag titles
    for sec in root_section.get_descendants():
        for word in sec.title.words:
            word.pos_tag = word_to_tag.get( word.word, word.pos_tag )
    
