import use_psyco, reader, config
import sys

def main():
    dataset_name = sys.argv[1]
    config.DATASET_NAME = dataset_name

    print "Loading corpus..."
    root_section = reader.read_corpus()
    compute_stats( root_section )

def bigrams( words ):
    for i in range(len(words) - 1):
        yield words[i].word + " " + words[i+1].word

def is_not_blank_section( sec ):
    return False == ( len( sec.title.words ) == 0 or len( sec.body ) == 0 )

def compute_stats( root ):
    sub_sections = filter( is_not_blank_section, root.get_descendants() )

    print "Computing unigram matches..."
    unigram_matches = 0
    unigram_total = 0
    for sec in sub_sections:
        for word in sec.title.words:
            if sec.has_unigram( word.word, False ):
                unigram_matches += 1
            unigram_total += 1

    print "Unigram matches: " + str( unigram_matches ) + "/" + str( unigram_total) \
          + " = " + str( 1.0 * unigram_matches / unigram_total )


    print "Computing bigram matches..."
    bigram_matches = 0
    bigram_total = 0
    for sec in sub_sections:
        text_bigrams = set()
        for sent in sec.body:
            for b in bigrams( sent.words ):
                text_bigrams.add( b )

        for tb in bigrams( sec.title.words ):
            if tb in text_bigrams:
                bigram_matches += 1
            bigram_total += 1

    print "Bigram matches: " + str( bigram_matches ) + "/" + str( bigram_total) \
          + " = " + str( 1.0 * bigram_matches / bigram_total )            


    print "Computing exact macthes..."
    exact_matches = 0
    exact_total = 0
    for sec in sub_sections:
        title_text = ' '.join( [ x.word for x in sec.title.words ] ) + ' '
        for sent in sec.body:
            sent_text = ' '.join( [ x.word for x in sent.words ] ) + ' '

            if title_text in sent_text:
                exact_matches += 1
                break
        exact_total += 1

    print "Exact matches: " + str( exact_matches ) + "/" + str( exact_total) \
          + " = " + str( 1.0 * exact_matches / exact_total )            
        
if __name__ == '__main__':
    sys.exit( main( ) )

