import config, porter_stemmer
import re, string, cPickle, os, sys

#----------------------------------------------------------------------------------
stemmer = porter_stemmer.PorterStemmer()

# a sequence number for use in cleaning the text of latex expressions.
expr_num = 0
# regular expressions to clean sentences of latex expressions & formatting characters
re_ampersand_all = re.compile("\&.*\&")
re_curlybracket_all = re.compile("\{.*\}")
re_alphapunc = re.compile("([A-Za-z\d]+[\\\\\/\+\=\%\^\*_]+[A-Za-z\d]*)|(([\\\\\/\+\=\%\^\*_]+[A-Za-z\d]+)[\\\\\/\+\=\%\^\*_]*)")
re_doublepunc = re.compile("([,.?\*!@\$%\^&])[\t ]*([,.?\*!@\$%\^&])")
re_emptybracket = re.compile("(\([^a-zA-Z0-9]*\))|(\[[^a-zA-Z0-9]*\])|(\<[^a-zA-Z0-9]*\>)")
re_multibracket = re.compile (">[>]+")
punctuation = ['.', ',', '!', '?', ';', ':']

#----------------------------------------------------------------------------------
def clean_sentence (text):
    
    global expr_num
    new_text = text.replace ('>th ', '> ')
    new_text = new_text.replace (' ax ', ' ')
    new_text = new_text.replace (' py ', ' ')
    new_text = re_ampersand_all.sub("<expr"+str(expr_num)+">", new_text)
    expr_num += 1
    new_text = re_curlybracket_all.sub("<expr"+str(expr_num)+">", new_text)
    expr_num += 1
    new_text = new_text.replace ('<expression>', '<expr' + str (expr_num)+'>')
    expr_num += 1
    new_text = new_text.replace ('<figure>', '<expr' + str (expr_num)+'>')
    expr_num += 1
    new_text = re_alphapunc.sub("", new_text)
    new_text = new_text.replace("\\", "")
    new_text = re_emptybracket.sub("", new_text)
    new_text = re_doublepunc.sub('\\2', new_text)
    new_text = re_multibracket.sub ('>', new_text)
   
    new_text = new_text.strip ()
    if len(new_text) > 0 and new_text[-1] not in punctuation: 
        new_text += "."

    return new_text

    
#----------------------------------------------------------------------------------
def tokenize_text (text):

    text = text.replace ('...', ' ... ')

    pattern = re.compile ('([,;:@#$%&])')
    text = pattern.sub (' \\1 ', text);

    # Assume sentence tokenization has been done first, so split FINAL periods only.
    pattern = re.compile ('([^.])([.])([]\\)}>\"\']*)[ \t]*$')
    text = pattern.sub ('\\1 \\2\\3 ', text);

    # however, we may as well split ALL question marks and exclamation points,
    # since they shouldn't have the abbrev.-marker ambiguity problem
    pattern = re.compile ('([?!])')
    text = pattern.sub (' \\1 ', text);

    # parentheses, brackets, etc.
    pattern = re.compile ('([\]\[\(\){}])')
    text = pattern.sub (' \\1 ', text);

    text = text.replace ('<', ' <');
    text = text.replace ('>', '> ');
    text = text.replace ('--', ' -- ');

    text = text.replace ('-', ' ')

    # Some taggers, such as Adwait Ratnaparkhi's MXPOST, use the parsed-file version of these symbols.
    # UNCOMMENT THE FOLLOWING 6 LINES if you're using MXPOST.
    # s/(/-LRB-/g
    text = text.replace ('(', '-LRB-')
    # s/)/-RRB-/g
    text = text.replace (')', '-RRB-')
    # s/\[/-LSB-/g
    text = text.replace ('[', '-LSB-')
    # s/\]/-RSB-/g
    text = text.replace (']', '-RSB-')
    # s/{/-LCB-/g
    text = text.replace ('{', '-LCB-')
    # s/}/-RCB-/g
    text = text.replace ('}', '-RCB-')

    # First off, add a space to the beginning and end of each line, to reduce
    # necessary number of regexps.
    text = ' ' + text + ' '
    text = text.replace ('"', " '' ");

    # possessive or close-single-quote
    pattern = re.compile ("([^'])' ")
    text = pattern.sub ("\1 ' ", text);

    # as in it's, I'm, we'd
    pattern = re.compile ("'([sSmMdD])")
    text = pattern.sub (" '\1 ", text);

    text = text.replace ("'ll ", " 'll ")
    text = text.replace ("'LL ", " 'LL ")
    text = text.replace ("'re ", " 're ")
    text = text.replace ("'RE ", " 'RE ")
    text = text.replace ("'ve ", " 've ")
    text = text.replace ("'VE ", " 'VE ")
    text = text.replace ("n't ", " n't ")
    text = text.replace ("N'T ", " N'T ")

    pattern = re.compile (" ([Cc])annot ")
    text = pattern.sub (" \1an not ", text);
    pattern = re.compile (" ([Dd])'ye ")
    text = pattern.sub (" \1id you ", text);
    pattern = re.compile (" ([Gg])imme ")
    text = pattern.sub (" \1ive me ", text);
    #text = pattern.sub (" \1im me ", text);
    pattern = re.compile (" ([Gg])onna ")
    text = pattern.sub (" \1oing to ", text);
    #text = pattern.sub (" \1on na ", text);
    pattern = re.compile (" ([Gg])otta ")
    text = pattern.sub (" \1ot to ", text);
    #text = pattern.sub (" \1ot ta ", text);
    pattern = re.compile (" ([Ll])emme ")
    text = pattern.sub (" \1et me ", text);
    #text = pattern.sub (" \1em me ", text);
    pattern = re.compile (" ([Mm])ore'n ")
    text = pattern.sub (" \1ore than ", text);
    #text = pattern.sub (" \1ore 'n ", text);
    text = text.replace (" 'Tis ", " It is ");
    text = text.replace (" 'tis ", " it is ");
    #pattern = re.compile (" '([Tt])is ")
    #text = pattern.sub (" \1 is ", text);
    text = text.replace (" 'Twas ", " It was ");
    text = text.replace (" 'twas ", " it was ");
    #pattern = re.compile (" '([Tt])was ")
    #text = pattern.sub (" \1 was ", text);
    pattern = re.compile (" ([Ww])anna ")
    text = pattern.sub (" \1ant to ", text);
    #text = pattern.sub (" \1an na ", text);

    text = text.replace ('  ', ' ')
    #if text[0] == ' ':
    #	text = text[:1]

    return text


#----------------------------------------------------------------------------------
def stem_word (word):
	return stemmer.stem (word, 0, len (word) - 1)


#----------------------------------------------------------------------------------
def split_into_words (text):
	words = text.split()

	proper_words = []
	for word in words:
		bHasPunctuation = False
		for ch in word:
			if ch not in string.digits and ch not in string.letters:
				bHasPunctuation = True
				break

		if not bHasPunctuation:
			proper_words.append (word)

		# bPunctuationOnly = True
		# for ch in word:
		#	if ch in string.digits or ch in string.letters:
		#		bPunctuationOnly = False
		#		break

		#if not bPunctuationOnly:
		#	proper_words.append (word)

	return proper_words

#----------------------------------------------------------------------------------
def split_into_sentences (text, sentence_delimiter):
	delimiter_pattern = re.compile (sentence_delimiter)
	return delimiter_pattern.split (text)


#----------------------------------------------------------------------------------
def scale_real_to_integer (number):
	return int (number * 10000000000)


#----------------------------------------------------------------------------------
def p (text):
	print '-- ' + str (text)
	sys.stdout.flush()

#----------------------------------------------------------------------------------
def save_section (sec):
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.section.pkl'
	f = open (file_name, 'wb')
	cPickle.dump (sec, f, -1)
	f.close ()


#----------------------------------------------------------------------------------
def load_section ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.section.pkl'
	f = open (file_name, 'rb')
	sec = cPickle.load (f)
	f.close ()

	return sec


#----------------------------------------------------------------------------------
def delete_section_pickle ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.section.pkl'
	if os.access (file_name + '.4', os.F_OK):
		os.unlink (file_name + '.4')
	if os.access (file_name + '.3', os.F_OK):
		os.rename (file_name + '.3', file_name + '.4')
	if os.access (file_name + '.2', os.F_OK):
		os.rename (file_name + '.2', file_name + '.3')
	if os.access (file_name + '.1', os.F_OK):
		os.rename (file_name + '.1', file_name + '.2')
	if os.access (file_name, os.F_OK):
		os.rename (file_name, file_name + '.1')
	#if os.access (file_name, os.F_OK):
	#	os.unlink (file_name)


#----------------------------------------------------------------------------------
def save_lm (lm):
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.lm.pkl'
	f = open (file_name, 'wb')
	cPickle.dump (lm, f, -1)
	f.close ()


#----------------------------------------------------------------------------------
def load_lm ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.lm.pkl'
	f = open (file_name, 'rb')
	lm = cPickle.load (f)
	f.close ()

	return lm


#----------------------------------------------------------------------------------
def delete_lm_pickle ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.lm.pkl'
	if os.access (file_name, os.F_OK):
		os.unlink (file_name)


#----------------------------------------------------------------------------------
def save_tag_lm (tag_lm):
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.tag_lm.pkl'
	f = open (file_name, 'wb')
	cPickle.dump (tag_lm, f, -1)
	f.close ()


#----------------------------------------------------------------------------------
def load_tag_lm ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.tag_lm.pkl'
	f = open (file_name, 'rb')
	tag_lm = cPickle.load (f)
	f.close ()

	return tag_lm


#----------------------------------------------------------------------------------
def delete_tag_lm_pickle ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.tag_lm.pkl'
	if os.access (file_name, os.F_OK):
		os.unlink (file_name)


#----------------------------------------------------------------------------------
def save_stopwords_set (stopwords):
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.stopwords.text'
	f = open (file_name, 'wb')
	for stopword in stopwords: f.write( stopword + "\n")
	f.close ()

#----------------------------------------------------------------------------------
def load_stopwords_set ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.stopwords.text'
	f = open (file_name, 'r')
	stopwords = f.readlines()
	f.close ()

	return set( [ x.strip() for x in stopwords ] )

#----------------------------------------------------------------------------------
def save_stem_model ( s_model):
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.stem_model.pkl'
	f = open (file_name, 'wb')
	cPickle.dump (s_model, f, -1)
	f.close ()

#----------------------------------------------------------------------------------
def load_stem_model ():
    	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.stem_model.pkl'
	f = open (file_name, 'rb')
	s_model = cPickle.load (f)
	f.close ()

	return s_model

#----------------------------------------------------------------------------------
def delete_stopwords_set ():
	file_name = config.PICKLE_PATH + config.DATASET_NAME + '.stopwords.text'
	if os.access (file_name, os.F_OK): os.unlink (file_name)

#----------------------------------------------------------------------------------
def null( a ): return

#----------------------------------------------------------------------------------
def time():
	p = os.popen("date")
	s = p.read()
	p.close()
	return s
    
#----------------------------------------------------------------------------------
def sort_dict_by_values(d):
    """ Returns the keys of dictionary d sorted by their values """
    items=d.items()
    backitems=[ [v[1],v[0]] for v in items]
    backitems.sort()
    return [ backitems[i][1] for i in range(0,len(backitems))]


#----------------------------------------------------------------------------------

np_tags = set(["JJ", "JJS", "JJR", "NN", "NNS", "NNP", "NNPS"])

def extract_nps( sentence_text, sentence_words):

    global np_tags

    nps = []
    np = []
        
    #p ('extracting from : ' + sentence_text)
    text_words = sentence_text.lower().split(" ")
    sent_word_index = 0
    for text_word_index in range( len(text_words) ):
        if sent_word_index >= len( sentence_words ): 
            break

        # If the text word matches the word object
        if sentence_words[ sent_word_index ].word == text_words[ text_word_index ]:

            # If the part of speech tag belongs to a noun phrase
            if sentence_words[ sent_word_index ].pos_tag in np_tags:
                np.append( sentence_words[ sent_word_index ] )
            else:
               if len(np) > 1: 
                   nps.append( np )
               np = []
               
            sent_word_index += 1

        # If the text word is punctuation
        else:
            if len(np) > 1: 
                nps.append( np )
            np = []
            
    if len(np) > 1:
        nps.append( np )
    
    return nps


def words_to_string( words ):
    return ' '.join( [ x.word for x in words ] )


def find_last_start_token (lst, item):
    for i in range(len(lst)):
        if lst[i] != item:
            return max(0, i-1)


